#ifndef TTC_RADIO_H
#define TTC_RADIO_H

/*{ ttc_radio.h **********************************************************
 
                      The ToolChain
                      
   Device independent support for 
   radio transceivers
   
   Currently implemented architectures: cc1101

   
   written by Gregor Rebel 2012
   
   Reference:
      
   
   How it works
   
   Each makefile.100_board* can define one or more RADIOs being connected via 
   This is done via lines of this type:
     COMPILE_OPTS += -D<OPTION>=<VALUE>
   This sets compile option <OPTION> to value <VALUE>
   
   RADIOn are enumerated starting at index 1 (e.g. RADIO1, RADIO2, ..).
   Each RADIOn can be configured via these constants:
   TTC_RADIOn                <RADIO>                sets RADIO driver to use -> <RADIO>.c, <RADIO>.h
   TTC_RADIOn_MOSI           <GPIO_BANK>,<PIN_NO>   SPI port to use for Master Out Slave In pin
   TTC_RADIOn_MISO           <GPIO_BANK>,<PIN_NO>   SPI port to use for Master In Slave Out pin
   TTC_RADIOn_SCK            <GPIO_BANK>,<PIN_NO>   SPI port to use for Serial Clock pin
   TTC_RADIOn_NSS            <GPIO_BANK>,<PIN_NO>   SPI port to use for Slave Select pin
   TTC_RADIO3_I2C            <I2C>                  sets internal I2C device to use
   TTC_RADIO3_I2C_SCL        <GPIO_BANK>,<PIN_NO>   I2C port to use for SCL pin
   TTC_RADIO3_I2C_SDA        <GPIO_BANK>,<PIN_NO>   I2C port to use for SCL pin
   TTC_RADIO3_I2C_ADDR       <ADDRESS>              I2C slave address of transceiver to use
   TTC_RADIO3_I2C_FREQUENCY  <FREQUENCY>            I2C clock frequency
   TTC_RADIO3_I2C_MODE       
   
   Note: Instead of  "<GPIO_BANK>,<PIN_NO>", the PIN_P<x><n> macros can be used (defined for each uC).
   Example: "GPIOC,7" can be given as "PIN_PC7"
   
   # Example excerpt from makefile.100_board_xxx
   COMPILE_OPTS += -DTTC_RADIO1=em357_radio   # builtin radio in em357/ stm32w108

   COMPILE_OPTS += -DTTC_RADIO2=ttc_radio  # transceiver connected via SPI1
   COMPILE_OPTS += -DTTC_RADIO2_INDEX=1       # use first CC1101
   COMPILE_OPTS += -DTTC_RADIO2_MOSI=PIN_PA0  # SC2
   COMPILE_OPTS += -DTTC_RADIO2_MISO=PIN_PA1  # SC2
   COMPILE_OPTS += -DTTC_RADIO2_SCK=PIN_PA2   # SC2
   COMPILE_OPTS += -DTTC_RADIO2_NSS=PIN_PA7 

   COMPILE_OPTS += -DTTC_RADIO3=some_radio    # some transceiver connected via I2C
   COMPILE_OPTS += -DTTC_RADIO3_I2C_ADDR=0x23 # use this I2C address
   COMPILE_OPTS += -DTTC_RADIO3_I2C_FREQUENCY=100000
   COMPILE_OPTS += -DTTC_RADIO3_I2C_MODE=STANDARD
   COMPILE_OPTS += -DTTC_RADIO3_I2C_SCL=PIN_PB1
   COMPILE_OPTS += -DTTC_RADIO3_I2C_SDA=PIN_PB2

}*/
//{ Defines/ TypeDefs ****************************************************

//} Defines
//{ Includes *************************************************************

#include "ttc_basic.h"
#include "ttc_radio_types.h"

#ifdef TARGET_ARCHITECTURE_STM32
  #include "ttc_radio.h"
#endif

//} Includes
//{ Structures/ Enums 2 **************************************************

// architecture specific configuration data of single RADIO 
#ifdef TARGET_ARCHITECTURE_STM32
    typedef ttc_radio_architecture_t ttc_radio_architecture_t;
#endif

//} Structures/ Enums
//{ Global Variables *****************************************************



//} Global Variables
//{ Function prototypes **************************************************


/* returns amount of RADIO devices available on current uC
 * @return amount of available RADIO devices
 */
u8_t ttc_radio_get_max_index();

/* fills out given RADIO_Generic with default values for indexed RADIO
 * @param RADIO_Index     device index of RADIO to init (1..ttc_radio_get_max_index())
 * @param RADIO_Generic   pointer to struct ttc_radio_generic_t
 * @return  == 0:         *RADIO_Generic has been initialized successfully; != 0: error-code
 */
ttc_radio_errorcode_e ttc_radio_get_defaults(u8_t RADIO_Index, ttc_radio_generic_t* RADIO_Generic);

/* fills out given RADIO_Generic with maximum valid values for indexed RADIO
 * @param RADIO_Index     device index of RADIO to init (1..ttc_radio_get_max_index())
 * @param RADIO_Generic   pointer to struct ttc_radio_generic_t
 * @return  == 0:         *RADIO_Generic has been initialized successfully; != 0: error-code
 */
ttc_radio_errorcode_e ttc_radio_get_features(u8_t RADIO_Index, ttc_radio_generic_t* RADIO_Generic);

/* initializes single RADIO
 * @param RADIO_Index     device index of RADIO to init (1..ttc_radio_get_max_index() )
 * @param RADIO_Generic   filled out struct ttc_radio_generic_t
 * @param RADIO_Arch      empty structure which will be filled with architecture dependend data
 * @return                == 0: RADIO has been initialized successfully; != 0: error-code
 */
ttc_radio_errorcode_e ttc_radio_init(u8_t RADIO_Index, ttc_radio_generic_t* RADIO_Generic, ttc_radio_architecture_t* RADIO_Arch);

/* Send out given amount of raw bytes from buffer.
 * Will not interpret bytes in buffer.
 *
 * Note: RADIO must be initialized before!
 * Note: This function is partially thread safe (use it for each RADIO from one thread only)!
 * Note: This function blocks until all bytes have been sent!
 * 
 * @param RADIO_Index    device index of RADIO to init (1..ttc_radio_get_max_index() )
 * @param Buffer         memory location from which to read data
 * @param Amount         amount of bytes to send from Buffer[]
 * @return               == 0: Buffer has been sent successfully; != 0: error-code
 */
ttc_radio_errorcode_e ttc_radio_send_raw(u8_t RADIO_Index, const u8_t* Buffer, u16_t Amount);

/* Send out given bytes from buffer until first zero byte or MaxLength reached.
 *
 * Note: RADIO must be initialized before!
 * Note: This function is partially thread safe (use it for each RADIO from one thread only)!
 * Note: This function blocks until all bytes have been sent!
 * 
 * @param RADIO_Index    device index of RADIO to init (1..ttc_radio_get_max_index() )
 * @param Buffer         memory location from which to read data
 * @param MaxLength      no more than this amount of bytes will be send
 * @return               == 0: Buffer has been sent successfully; != 0: error-code
 */
ttc_radio_errorcode_e ttc_radio_send_string(u8_t RADIO_Index, const u8_t* Buffer, u16_t MaxLength);

/* Send out given data word (8 or 9 bits)
 *
 * Note: RADIO must be initialized before!
 * Note: This function is partially thread safe (use it for each RADIO from one thread only)!
 * Note: This function blocks until all bytes have been sent!
 *
 * @param RADIO_Index    device index of RADIO to init (1..ttc_radio_get_max_index() )
 * @param Byte           8 or 9 bit value to send
 * @return               == 0: Buffer has been sent successfully; != 0: error-code
 */
ttc_radio_errorcode_e ttc_radio_send_word(u8_t RADIO_Index, const u16_t Word);

/* Reads single data byte from input buffer.
 *
 * Note: RADIO must be initialized before!
 * Note: This function is partially thread safe (use it for each RADIO from one thread only)!
 * Note: This function blocks until a valid byte has been received or timeout occurs
 *
 * @param RADIO_Index    device index of RADIO to init (1..ttc_radio_get_max_index() )
 * @param Word           pointer to 16 bit buffer where to store Word
 * @param TimeOut        >0: max microseconds to wait; ==0: no timeout; ==-1: immediately return
 * @return               == 0: Word has been read successfully; != 0: error-code
 */
ttc_radio_errorcode_e ttc_radio_read_byte(u8_t RADIO_Index, u8_t* Byte, u16_t TimeOut);

/* Hands out pointer to oldest internal receive buffer.
 * Each radio has at least three recceiver buffers. If one is handed out, the buffer handed out 
 * before is automatically reused.
 * Use this function to get a pointer to the oldest received data. 
 *
 * Note: RADIO must be initialized before!
 * Note: This function is partially thread safe (use it for each RADIO from one thread only)!
 * Note: This function blocks until a valid byte has been received or timeout occurs
 *
 * @param RADIO_Index    device index of RADIO to init (1..ttc_radio_get_max_index() )
 * @param Buffer         will be loaded with pointer to receive buffer
 * @param Amount         will be loaded with amount of valid bytes in Buffer[]
 * @return               == 0: Word has been read successfully; != 0: error-code
 */
ttc_radio_errorcode_e ttc_radio_getbytes(u8_t RADIO_Index, u8_t** Buffer, u16_t* Amount);

/* Registers a function to be called whenever at least 1 byte or a special character has been received
 * 
 * RxFunction(u8_t RADIO_Index) will be called whenever the set signal mode is met. 
 * (see enum ttc_radio_signal_mode_e)
 * 
 * @param RADIO_Index    device index of RADIO to init (1..ttc_radio_get_max_index() )
 * @param RxFunction     points to function which will be called to process received data
 * @param EOM_Character  special character which identifies end of a message
 * @param Mode           specifies, when to call RxFunction() 
 * @return               == 0: function has been registered successfully; != 0: error-code
 */
ttc_radio_errorcode_e ttc_radio_register_rx_function(u8_t RADIO_Index, void*RxFunction(u8_t), u8_t EOM_Character, ttc_radio_signal_mode_e Mode);

//} Function prototypes

#endif //TTC_RADIO_H
